/**
 * @file
 * Site preview.
 */

(function ($, window, Drupal, once, drupalSettings) {
  $.fn.dialogToggleOnlyWhenCollapsed = () => {
    const dialogWidth = jQuery('#drupal-off-canvas').dialog('option', 'width');
    if (dialogWidth <= 440) {
      $.fn.dialogToggle();
    }
  };

  $.fn.dialogToggle = () => {
    const dialogWidth = jQuery('#drupal-off-canvas').dialog('option', 'width');
    const state = dialogWidth <= 440 ? 'collapsed' : 'expanded';
    const width = state === 'collapsed' ? '1200' : '440';
    const label = state === 'collapsed' ? '>>' : '<<';

    jQuery('#dialog-toggle').html(label);
    jQuery('#drupal-off-canvas').dialog('option', {
      width,
      resizable: false,
    });
  };

  $.fn.visualEditorOpenDialog = (url, dialog) => {
    const ajaxSettings = {
      url,
      dialogRenderer: 'off_canvas',
      dialogType: 'dialog',
      dialog,
    };

    Drupal.ajax(ajaxSettings).execute();
  };

  $.fn.visualEditorReload = (storage, id) => {
    if (storage === 'node' && id) {
      // Reload node after save to refresh form state and prevent error:
      // The content has either been modified by another user,
      // or you have already submitted modifications.
      // As a result, your changes cannot be saved.
      // Reload Drupal page to refresh form state.
      window.location.href = `/node/${id}`;
    }
  };

  Drupal.behaviors.visualEditorLoad = {
    attach(context) {
      const {
        openLoad,
        isPreview = false,
        node,
      } = drupalSettings.visual_editor;
      const { selector } = drupalSettings.decoupled_preview_iframe;
      const width = 440;
      // Attach dialog toggle event
      if (once('dialog-toggle', '#dialog-toggle', context).length) {
        jQuery('#dialog-toggle').bind('click', () => {
          $.fn.dialogToggle();
        });
      }
      // Body ready events
      once('visual_editor', selector, context).forEach(() => {
        if (openLoad && node) {
          // Open dialog
          $.fn.visualEditorOpenDialog(
            `/visual_editor/form/node/${node}/edit?preview=${isPreview}`,
            {
              width,
              resizable: false,
            },
          );
          // Remove resizable from dialog
          jQuery(window).on('dialog:aftercreate', function () {
            jQuery('#drupal-off-canvas').dialog('option', {
              width,
              resizable: false,
            });
          });
        }
      });
    },
  };

  Drupal.behaviors.visualEditorListener = {
    attach() {
      window.addEventListener(
        'message',
        (event) => {
          const { wrapper } = drupalSettings.visual_editor;
          const { data } = event;
          const { type, uuid } = data;

          if (!type || !uuid) {
            return;
          }

          if (type === 'VISUAL_EDITOR_COMPONENT') {
            const paragraph = $(`input[data-visual-editor-uuid="${uuid}"]`);
            const paragraphId = paragraph.attr('id');
            const paragraphSelector = jQuery(`#${paragraphId}`)
              .parent()
              .parent()
              .parent()
              .data('drupal-selector');

            // Expand paragraph
            paragraph.trigger('mousedown');
            // Expand dialog
            $.fn.dialogToggleOnlyWhenCollapsed();

            if (
              wrapper.type &&
              wrapper.mode &&
              wrapper.selector &&
              wrapper.position
            ) {
              if (wrapper.type === 'tab' && Number(wrapper.position) > 0) {
                const wrapperSelector = `${wrapper.selector} > .${
                  wrapper.mode
                }-${wrapper.type}s > .${wrapper.mode}-${
                  wrapper.type
                }s-list > .${wrapper.mode}-${wrapper.type}-button-${
                  Number(wrapper.position) - 1
                }`;
                $(wrapperSelector).children(0).click();
              }
            }

            // @todo: find why scrollIntoView got reset
            // Scroll to paragraph
            if (paragraphSelector) {
              const selector = `[data-drupal-selector="${paragraphSelector}"]`;
              document.querySelector(selector).scrollIntoView({
                behavior: 'auto',
                block: 'center',
                inline: 'nearest',
              });
            }

            event.stopImmediatePropagation();
          }
        },
        false,
      );
    },
  };
})(jQuery, window, Drupal, once, drupalSettings);
