<?php

declare(strict_types=1);

namespace Drupal\visual_editor;

/**
 * Trait for \Drupal\visual_editor\Form\NodeEditForm.
 */
trait FormEditTrait {

  /**
   * Sets the entity for sidebar edit form.
   *
   * @param array $form
   *   The form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface
   *   The node object to return.
   */
  public function calculateNodeEntity(&$form, &$form_state) {
    $query = \Drupal::request()->query;
    $is_preview = $query->has('preview') && $query->get('preview') == 'true';

    // Try to restore from temp store, this must be done before calling
    // parent::form().
    $store = $this->tempStoreFactory->get('node_preview');
    $node_uuid = $form_state->getFormObject()->getEntity()->uuid();

    if ($is_preview && !$form_state->isRebuilding() && $node_uuid && $preview = $store->get($node_uuid)) {
      /** @var \Drupal\Core\Form\FormStateInterface $preview */
      $form_state->setStorage($preview->getStorage());
      $form_state->setUserInput($preview->getUserInput());

      // Rebuild the form.
      $form_state->setRebuild();

      // The combination of having user input and rebuilding the form means
      // that it will attempt to cache the form state which will fail if it is
      // a GET request.
      $form_state->setRequestMethod('POST');

      $this->entity = $preview->getFormObject()->getEntity();
      $this->entity->in_preview = NULL;

      $form_state->set('has_been_previewed', TRUE);

      return $this->entity;
    }
    else {
      $this->entity = $form_state->getFormObject()->getEntity();
      return $form_state->getFormObject()->getEntity();
    }

  }

}
